function edgeawaredemo1d

% create random signal with plateaus
sig=[];
for c1=[1 5 0 -3 2]
  sig=[sig;randn(3+ceil(7*rand),1)*0.2+c1];
end

% do various filtering techniques to compare
cmp=ji1d(4,sig);
[dns,dom,itr]=ji1ep(4,sig);
smp=[1:dom(end)]';
res=ji1d(4,itr);

% plot results
subplot(1,2,1)
plot(sig,'r-o')
hold on
plot(cmp,'g')
plot(dns,'b')
hold off  
axis tight
xlabel('original domain')
legend('input','uniform smoothing','edge-aware smoothing')

subplot(1,2,2)
plot(smp,itr,'r-o')
hold on
plot(res,'g')
hold off
axis tight
xlabel('extended domain')
legend('resampled input','uniform smoothing')
  
set(1,'Color',[1 1 1])
%export_fig('out/edgeaware.png')

function [res,dom,itr]=ji1ep(w,sig,dom)
  if nargin==2
    dom=cumsum(1+[0;abs(diff(sig))]); % has numel of signal
  end
  dom(end)=ceil(dom(end)); % the error introduced by rounding up at the border should be negligable
  smp=[1:dom(end)]';
  itr=interp1(dom,sig,smp);
  res=interp1(smp,ji1d(w,itr),dom);

function res=ji1d(w,sig)
  w=ceil(w)*2+1;
  x=linspace(-2,2,w)';
  fil=sqrt(1/pi)*exp(-x.*x); %fil=sqrt(a/pi)*exp(-a*x.*x);
  fil=fil/sum(fil);  
  num=numel(sig);
  f_sig=fft(sig);
  f_fil=fft(fil,num);
  res=ifft(f_sig.*f_fil);
  res=real(res);    
  res=res([(w+1)/2:end 1:(w-1)/2]); % rotate  
